#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>		/* normal stuff             */
#include <ctype.h>
#include <sys/utsname.h>
#include <sys/times.h>

/* ensure getopt */
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* ensure sys/types */
#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif

/* some systems need this */
#ifdef HAVE_MATH_H
#include <math.h>
#endif

/* SunOS 4.x Fix */
#ifndef CLK_TCK
#define CLK_TCK _SC_CLK_TCK
#endif

#include "parser.h"
#include "linklist.h"
#include "hashtab.h"
#include "botvisits.h"
#include "monthlydata.h"

#define msg_nomem_sc  "no memory"
#define msg_bad_date "bad date"


int verbose = 0;
int debug_mode = 0;

int cur_year = 0, cur_month = 0,	/* year/month/day/hour      */
  cur_day = 0, cur_hour = 0,	/* tracking variables       */
  cur_min = 0, cur_sec = 0;


int rec_year, rec_month = 1, rec_day, rec_hour, rec_min, rec_sec;


/* month names used for parsing logfile (shouldn't be lang specific) */
char *log_month[12] = { "jan", "feb", "mar",
  "apr", "may", "jun",
  "jul", "aug", "sep",
  "oct", "nov", "dec"
};
int
parse_date ()
{
  return 0;
}

void
clear_month ()
{
  del_htabs ();
  botlist_clearall();
}

void
check_bot()
{
  unsigned char *name;

  /* Check if search engine referrer or return  */
  if ((name = isinglist (bot_useragents, log_rec.agent)) == NULL)
    return;

  botlist_updateday(name, rec_day);

//  printf("%s %s\n", cps, log_rec.agent);
}


/*********************************************/
/* SRCH_STRING - get search strings from ref */
/*********************************************/

void
srch_string (char *ptr, char *pageurl)
{
  /* ptr should point to unescaped query string */
  char tmpbuf[BUFSIZE];
  char srch[80] = "";
  unsigned char *cp1, *cp2, *cps;
  int sp_flg = 0;
  unsigned char ch;
  char *kw, *str;

  /* Check if search engine referrer or return  */
  if ((cps = isinglist (search_list, log_rec.refer)) == NULL)
    return;

  /* Try to find query variable */
  srch[0] = '?';
  strcpy (&srch[1], cps);	/* First, try "?..."      */
  if ((cp1 = strstr (ptr, srch)) == NULL)
    {
      srch[0] = '&';		/* Next, try "&..."       */
      if ((cp1 = strstr (ptr, srch)) == NULL)
	return;			/* If not found, split... */
    }
  cp2 = tmpbuf;
  while (*cp1 != '=' && *cp1 != 0)
    cp1++;
  if (*cp1 != 0)
    cp1++;
  while (*cp1 != '&' && *cp1 != 0)
    {
      if (*cp1 == '"' || *cp1 == ',' || *cp1 == '?')
	{
	  cp1++;
	  continue;
	}			/* skip bad ones..    */
      else
	{
	  if (*cp1 == '+')
	    *cp1 = ' ';		/* change + to space  */
	  if (sp_flg && *cp1 == ' ')
	    {
	      cp1++;
	      continue;
	    }			/* compress spaces    */
	  if (*cp1 == ' ')
	    sp_flg = 1;
	  else
	    sp_flg = 0;		/* (flag spaces here) */
	  *cp2++ = tolower (*cp1);	/* normal character   */
	  cp1++;
	}
    }
  *cp2 = 0;
  cp2 = tmpbuf;
  if (tmpbuf[0] == '?')
    tmpbuf[0] = ' ';		/* format fix ?       */
  while (*cp2 != 0 && isspace (*cp2))
    cp2++;			/* skip leading sps.  */
  if (*cp2 == 0)
    return;

  /* any trailing spaces? */
  cp1 = cp2 + strlen (cp2) - 1;
  while (cp1 != cp2)
    if (isspace (*cp1))
      *cp1-- = '\0';
    else
      break;

  /* strip invalid chars */
  cp1 = cp2;
  while (*cp1 != 0)
    {
      if ((*cp1 < 32) || (*cp1 == 127))
	*cp1 = '_';
      cp1++;
    }

  if (put_snode (cp2, (u_long) 1, sr_htab, pageurl))
    {
      if (verbose)
	/* Error adding search string node, skipping .... */
	fprintf (stderr, "%s %s\n", msg_nomem_sc, tmpbuf);
    }

  if (put_pnode (pageurl, (u_long) 1, pg_htab, cp2))
    {
      if (verbose)
	/* Error adding search string node, skipping .... */
	fprintf (stderr, "%s %s\n", msg_nomem_sc, tmpbuf);
    }


  /* Divide to keywords */
  str = kw = cp2;

  if (*str)
    do
      {
	if (*str == ' ' || *str == 0)
	  {
	    ch = *str;
            *str = 0;
            if (*kw != 0)
	    if (put_snode (kw, (u_long) 1, kw_htab, pageurl))
	      {
		if (verbose)
		  /* Error adding search string node, skipping .... */
		  fprintf (stderr, "%s %s\n", msg_nomem_sc, tmpbuf);
	      }
            *str = ch;
	    if (*str) 
              kw = str+1;
	  }

	str++;

      }
    while (*str);


  return;
}

void
main (int argc, char **argv)
{
  FILE *log_fp;
  char logline[0x400];
  int total_rec, total_bad;
  char *log_fname = "../access_log";
  int i;
  unsigned char *cp1, *cp2;

   extern char *optarg;                  /* used for command line       */
   extern int optind;                    /* parsing routine 'getopt'    */
   extern int opterr;


   /* get command line options */
   opterr = 0;     /* disable parser errors */
   while ((i=getopt(argc,argv,"c:"))!=EOF)
   {
      switch (i)
      {
        case 'c': /*get_config(optarg);      */  break;  /* Config file         */
      }
   }

   if (argc - optind != 0) log_fname = argv[optind];

  total_rec = 0;
  total_bad = 0;

  clear_month();

  if (bot_useragents == NULL)
    {
      /* If no search engines defined, define some :) */
      add_glist ("Googlebot/      Google", &bot_useragents);
      add_glist ("Googlebot-image/  Google-Image", &bot_useragents);
      add_glist ("Slurp          Yahoo", &bot_useragents);
      add_glist ("SeznamBot/      Seznam.CZ", &bot_useragents);
      add_glist ("msnbot/        MSN", &bot_useragents);
      add_glist ("msnbot-media/        MSN-Media", &bot_useragents);
    }


  if (search_list == NULL)
    {
      /* If no search engines defined, define some :) */
      add_glist ("yahoo.com      p=", &search_list);
      add_glist ("altavista.com  q=", &search_list);
      add_glist ("google.com     q=", &search_list);
      add_glist ("google.cz     q=", &search_list);
      add_glist ("google.de     q=", &search_list);
      add_glist ("google.fr     q=", &search_list);
      add_glist ("google.co.uk     q=", &search_list);
      add_glist ("google.it     q=", &search_list);
      add_glist ("google.es     q=", &search_list);
      add_glist ("google.ca     q=", &search_list);
      add_glist ("google.pl     q=", &search_list);
      add_glist ("google.co.kr     q=", &search_list);
      add_glist ("google.ru     q=", &search_list);
      add_glist ("google.nl     q=", &search_list);
      add_glist ("google.com.mx     q=", &search_list);
      add_glist ("google.ro     q=", &search_list);
      add_glist ("google.co.jp     q=", &search_list);
      add_glist ("google.ch     q=", &search_list);
      add_glist ("google.com.tr     q=", &search_list);
      add_glist ("google.fi     q=", &search_list);
      add_glist ("google.hi     q=", &search_list);
      add_glist ("google.be     q=", &search_list);
      add_glist ("google.com.ar     q=", &search_list);
      add_glist ("google.co.in     q=", &search_list);
      add_glist ("google.com.ua     q=", &search_list);
      add_glist ("eureka.com     q=", &search_list);
      add_glist ("lycos.com      query=", &search_list);
      add_glist ("hotbot.com     MT=", &search_list);
      add_glist ("msn.com        MT=", &search_list);
      add_glist ("infoseek.com   qt=", &search_list);
      add_glist ("webcrawler searchText=", &search_list);
      add_glist ("excite         search=", &search_list);
      add_glist ("netscape.com   search=", &search_list);
      add_glist ("mamma.com      query=", &search_list);
      add_glist ("alltheweb.com  query=", &search_list);
      add_glist ("northernlight.com qr=", &search_list);
    }

  log_fp = fopen (log_fname, "rt");
  if (log_fp == NULL) {
   perror(log_fname);
   exit(1);
  }

  while (fgets (logline, sizeof (logline), log_fp))
    {
      total_rec++;
      if (strlen (logline) == (sizeof (logline) - 1))
	{
	  total_bad++;		/* bump bad record counter      */

	  /* get the rest of the record */
	  while ((fgets (logline, sizeof (logline), log_fp) != NULL))
	    {
	      if (strlen (logline) < sizeof (logline) - 1)
		{
		  break;
		}
	    }
	  continue;		/* go get next record if any    */
	}

      if (parse_record_web (logline) == 0)
	{
	  continue;
	}


      int i;

      /* convert month name to lowercase */
      for (i = 4; i < 7; i++)
	log_rec.datetime[i] = tolower (log_rec.datetime[i]);

      /* get year/month/day/hour/min/sec values    */
      for (i = 0; i < 12; i++)
	{
	  if (strncmp (log_month[i], &log_rec.datetime[4], 3) == 0)
	    {
	      rec_month = i + 1;
	      break;
	    }
	}

      rec_year = atoi (&log_rec.datetime[8]);	/* get year number (int)   */
      rec_day = atoi (&log_rec.datetime[1]);	/* get day number          */
      rec_hour = atoi (&log_rec.datetime[13]);	/* get hour number         */
      rec_min = atoi (&log_rec.datetime[16]);	/* get minute number       */
      rec_sec = atoi (&log_rec.datetime[19]);	/* get second number       */

      /* Kludge for Netscape server time (0-24?) error                   */
      if (rec_hour > 23)
	rec_hour = 0;

      /* minimal sanity check on date */
      if ((i >= 12) || (rec_min > 59) || (rec_sec > 59) || (rec_year < 1990))
	{
	  total_bad++;		/* if a bad date, bump counter      */
	  if (verbose)
	    {
	      fprintf (stderr, "%s: %s [%lu]",
		       msg_bad_date, log_rec.datetime, total_rec);
	      fprintf (stderr, "\n");
	    }
	  continue;		/* and ignore this record           */
	}


         /* fix URL field */
         cp1 = cp2 = log_rec.url;
         /* handle null '-' case here... */
         if (*++cp1 == '-') { *cp2++ = '-'; *cp2 = '\0'; }
         else
         {
            /* strip actual URL out of request */
            while  ( (*cp1 != ' ') && (*cp1 != '\0') ) cp1++;
            if (*cp1 != '\0')
            {
               /* scan to begin of actual URL field */
               while ((*cp1 == ' ') && (*cp1 != '\0')) cp1++;
               /* remove duplicate / if needed */
               if (( *cp1=='/') && (*(cp1+1)=='/')) cp1++;
               while ((*cp1 != ' ')&&(*cp1 != '"')&&(*cp1 != '\0'))
                  *cp2++ = *cp1++;
               *cp2 = '\0';
            }
         }



      /* first time through? */
      if (cur_month == 0)
	{
	  /* if yes, init our date vars */
	  cur_month = rec_month;
	  cur_year = rec_year;
	  cur_day = rec_day;
	  cur_hour = rec_hour;
	  cur_min = rec_min;
	  cur_sec = rec_sec;
	  append_month(rec_year,rec_month);
	  //f_day=rec_day;
	}

      /* PROCESS RECORD */

      if (cur_month != rec_month)
	{
	  /* if yes, do monthly stuff */
	  //t_visit=tot_visit(sm_htab);
	  //month_update_exit(req_tstamp);    /* process exit pages      */
	  write_month_html ();	/* generate HTML for month */
	  clear_month ();
	  cur_month = rec_month;	/* update our flags        */
	  cur_year = rec_year;
	  append_month(rec_year,rec_month);

	  //f_day=l_day=rec_day;
	}

      srch_string (log_rec.refer, log_rec.url);
      check_bot ();
    }

  write_month_html ();		/* generate HTML for month */

  write_index_html();

  fclose (log_fp);
}
